LazyLinearMemory

Struct LazyLinearMemory 

pub struct LazyLinearMemory { /* private fields */ }
A linear memory wrapper that materializes its inner backend on first access.

If the wrapped backend fails to create the inner memory during first access, this wrapper will panic.

Implementations

Source

impl LazyLinearMemory

pub fn try_new(ty: MemoryType, backend: MemoryBackend) -> Result<Self>

Creates a lazy memory for ty using backend for the eventual materialized storage.